#include <string>
#include <iostream>
#include <vector>

#include <boost/range/begin.hpp>
#include <boost/range/end.hpp>
#include <boost/utility/enable_if.hpp>
#include <boost/type_traits/is_same.hpp>

//	Iterator-based versions
template<class ForwardIterator, class Compare>
std::pair<ForwardIterator, ForwardIterator>
Foo ( ForwardIterator first, ForwardIterator last, Compare comp ) {
	return std::make_pair<ForwardIterator, ForwardIterator> (first, last);
	}

template<class ForwardIterator>
std::pair<ForwardIterator, ForwardIterator>
Foo ( ForwardIterator first, ForwardIterator last ) {
	return Foo ( first, last, 
		std::less<typename std::iterator_traits<ForwardIterator>::value_type> ());
	}

//	Range-based versions

namespace detail {
	template <typename Range>
	struct range_pair {
		typedef std::pair<typename boost::range_iterator<const Range>::type, 
		                  typename boost::range_iterator<const Range>::type> type;
		};
}

template<class Range>
typename detail::range_pair<Range>::type
Foo ( const Range &r )
{
    return Foo (boost::begin (r), boost::end (r));
}


template<class Range, class Compare>
typename boost::lazy_disable_if_c<
		boost::is_same<Range, Compare>::value,
		typename detail::range_pair<Range> >
	::type
Foo ( const Range &r, Compare comp )
{
    return Foo (boost::begin (r), boost::end (r), comp );
}


bool comp ( int x ) { return x > 10; }

/*
template <typename Container>
void test_sequence_no_comparison ( Container &v ) {

	typedef std::pair <typename Container::iterator, typename Container::iterator> Pair;

	Pair res = Foo ( v.begin (), v.end ());

	std::reverse ( v.begin (), v.end ());
	res = Foo ( v.begin (), v.end ());
	
	std::random_shuffle ( v.begin (), v.end ());
	res = Foo ( v.begin (), v.end ());
	
//	One element
	int v0 = v[0];
	res = Foo ( v.begin (), v.begin () + 1 );
	
//	Two elements
	int v1 = v[1];
	res = Foo ( v.begin (), v.begin () + 2 );
	
//	With comparator
	res = Foo ( v, comp );
	}
*/

void test_sequence_no_comparison ( std::vector<int> &v ) {

	typedef std::pair <std::vector<int>::iterator, std::vector<int>::iterator> Pair;

	Pair res = Foo ( v.begin (), v.end ());

	std::reverse ( v.begin (), v.end ());
	res = Foo ( v.begin (), v.end ());
	
	std::random_shuffle ( v.begin (), v.end ());
	res = Foo ( v.begin (), v.end ());
	
//	One element
	int v0 = v[0];
	res = Foo ( v.begin (), v.begin () + 1 );
	
//	Two elements
	int v1 = v[1];
	res = Foo ( v.begin (), v.begin () + 2 );
	
//	With comparator
	res = Foo ( v, comp );
	}

int main ( int argc, char *argv [] ) {
	std::vector<int> v;
	
	v.clear ();
	for ( int i = 5; i < 15; ++i )
		v.push_back ( i );
	test_sequence_no_comparison ( v );	// even # of elements
	
	return 0;
	}
